@@ -1,5 +1,5 @@
 // use crate::errors::{ErrorKind, Result};
-use crate::client::tx_types::TxFeeResult;
+use crate::client::tx_types::{TXResultAsync, TXResultSync, TxFeeResult};
 use crate::core_types::{Coin, StdFee, StdSignMsg, StdSignature};
 use reqwest::header::{HeaderMap, CONTENT_TYPE, USER_AGENT};
 use reqwest::{Client, RequestBuilder};
@@ -33,11 +33,14 @@ mod wasm;
 mod wasm_types;
 
 use crate::errors::TerraRustAPIError;
+use crate::errors::TerraRustAPIError::TxResultError;
 use crate::messages::Message;
+use crate::AddressBook;
 use crate::PrivateKey;
 use bitcoin::secp256k1::{All, Secp256k1};
 use futures::TryFutureExt;
 use rust_decimal_macros::dec;
+use std::fs::File;
 
 /// Version # of package sent out on requests to help with debugging
 const VERSION: Option<&'static str> = option_env!("CARGO_PKG_VERSION");
@@ -99,6 +102,9 @@ pub struct Terra<'a> {
     pub debug: bool,
 }
 impl<'a> Terra<'a> {
+    const NETWORK_PROD_ADDRESS_BOOK: &'a str = "https://network.terra.dev/addrbook.json";
+    const NETWORK_TEST_ADDRESS_BOOK: &'a str = "https://network.terra.dev/testnet/addrbook.json";
+
     /// Create a FULL client interface
     pub async fn lcd_client(
         url: &'a str,
@@ -164,7 +170,7 @@ impl<'a> Terra<'a> {
         wasm::Wasm::create(self)
     }
 
-    fn construct_headers() -> HeaderMap {
+    pub fn construct_headers() -> HeaderMap {
         let mut headers = HeaderMap::new();
 
         headers.insert(
@@ -350,6 +356,60 @@ impl<'a> Terra<'a> {
             .await?
             .await?
     }
+    /// helper: sign & submit the transaction sync
+    pub async fn submit_transaction_sync(
+        &self,
+        secp: &Secp256k1<All>,
+        from: &'a PrivateKey,
+        messages: &'a [Message],
+        memo: Option<String>,
+    ) -> anyhow::Result<TXResultSync> {
+        let (std_sign_msg, sigs) = self
+            .generate_transaction_to_broadcast(secp, from, messages, memo)
+            .await?;
+        let resp = self.tx().broadcast_sync(&std_sign_msg, &sigs).await?;
+
+        match resp.code {
+            Some(code) => Err(TxResultError(code, resp.txhash, resp.raw_log).into()),
+            None => Ok(resp),
+        }
+    }
+    /// helper: sign & submit the transaction async
+    pub async fn submit_transaction_async(
+        &self,
+        secp: &Secp256k1<All>,
+        from: &'a PrivateKey,
+        messages: &'a [Message],
+        memo: Option<String>,
+    ) -> anyhow::Result<TXResultAsync> {
+        let (std_sign_msg, sigs) = self
+            .generate_transaction_to_broadcast(secp, from, messages, memo)
+            .await?;
+        let resp = self.tx().broadcast_async(&std_sign_msg, &sigs).await?;
+        Ok(resp)
+    }
+
+    /// fetch the address book for the production network
+    pub async fn production_address_book() -> anyhow::Result<AddressBook> {
+        Self::address_book(Self::NETWORK_PROD_ADDRESS_BOOK).await
+    }
+    /// fetch the address book for the testnet network
+    pub async fn testnet_address_book() -> anyhow::Result<AddressBook> {
+        Self::address_book(Self::NETWORK_TEST_ADDRESS_BOOK).await
+    }
+    /// fetch a address book json structure
+    pub async fn address_book(addr_url: &str) -> anyhow::Result<AddressBook> {
+        if let Some(file_name) = addr_url.strip_prefix("file://") {
+            let file = File::open(file_name).unwrap();
+            let add: AddressBook = serde_json::from_reader(file)?;
+            Ok(add)
+        } else {
+            let client = reqwest::Client::new();
+
+            let req = client.get(addr_url).headers(Self::construct_headers());
+            Self::resp::<AddressBook>(addr_url, req).await
+        }
+    }
 }
 #[cfg(test)]
 mod tst {
@@ -361,7 +421,7 @@ mod tst {
 
     #[test]
     pub fn test_send() -> anyhow::Result<()> {
-        let str_1 =  "island relax shop such yellow opinion find know caught erode blue dolphin behind coach tattoo light focus snake common size analyst imitate employ walnut";
+        let str_1 = "island relax shop such yellow opinion find know caught erode blue dolphin behind coach tattoo light focus snake common size analyst imitate employ walnut";
         let secp = Secp256k1::new();
         let pk = PrivateKey::from_words(&secp, str_1)?;
         let pub_k = pk.public_key(&secp);
@@ -405,7 +465,7 @@ mod tst {
 
     #[test]
     pub fn test_wasm() -> anyhow::Result<()> {
-        let key_words="sell raven long age tooth still predict idea quit march gasp bamboo hurdle problem voyage east tiger divide machine brain hole tiger find smooth";
+        let key_words = "sell raven long age tooth still predict idea quit march gasp bamboo hurdle problem voyage east tiger divide machine brain hole tiger find smooth";
         let secp = Secp256k1::new();
         let private = PrivateKey::from_words(&secp, key_words)?;
         let public_key = private.public_key(&secp);
@@ -450,4 +510,21 @@ mod tst {
         assert_eq!(js_sig, js_sig_eq);
         Ok(())
     }
+
+    #[tokio::test]
+    pub async fn test_address_book() -> anyhow::Result<()> {
+        let prod = Terra::production_address_book().await?;
+        assert!(prod.addrs.len() > 0);
+        let test = Terra::testnet_address_book().await?;
+        assert!(test.addrs.len() > 0);
+        let file_version = Terra::address_book("file://resources/addressbook.json").await?;
+        assert_eq!(file_version.key, "775cf30a073ca5e97fb07a00");
+        assert!(file_version.addrs.len() > 1);
+        assert_eq!(
+            file_version.addrs[0].addr.id,
+            "ebca6b5d3cc2da9dfdfe4b1c045043fce686f143"
+        );
+
+        Ok(())
+    }
 }
